from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.urls import reverse

import pytest

from organizations.models import Organization
from organizations.models import OrganizationUser
from organizations.utils import create_organization
from organizations.views import base
from test_accounts.models import Account
from test_accounts.models import AccountUser
from tests.utils import request_factory_login


@pytest.fixture
def account_user():
    yield User.objects.create(username="AccountUser", email="akjdkj@kjdk.com")


@pytest.fixture
def account_account(account_user):
    yield create_organization(account_user, "Acme", org_model=Account)


@pytest.fixture
def org_organization(account_user):
    yield create_organization(account_user, "Acme", org_model=Organization)


@pytest.fixture
def extra_org_user(org_organization):
    new_user = User.objects.create(username="NotYou", email="not@you.com")
    org_organization.add_user(new_user)
    yield new_user


@pytest.fixture
def invitee_user():
    yield User.objects.create_user(
        "newmember", email="jd@123.com", password="password123"
    )


class TestUserReminderView:
    @pytest.mark.parametrize("method", [("get"), ("post")])
    def test_bad_request_for_active_user(
        self, rf, account_account, account_user, invitee_user, method
    ):
        class OrgUserReminderView(base.BaseOrganizationUserRemind):
            org_model = Account
            user_model = AccountUser

        request = getattr(rf, method)("/", user=account_user)
        kwargs = {"organization_pk": account_account.pk, "user_pk": account_user.pk}
        response = OrgUserReminderView.as_view()(request, **kwargs)
        assert response.status_code == 410

    def test_organization_user_reminder(
        self, rf, account_account, account_user, invitee_user
    ):
        class OrgUserReminderView(base.BaseOrganizationUserRemind):
            org_model = Account
            user_model = AccountUser

        invitee_user.is_active = False
        invitee_user.save()
        account_account.add_user(invitee_user)
        request = rf.get("/", user=account_user)
        kwargs = {"organization_pk": account_account.pk, "user_pk": invitee_user.pk}
        response = OrgUserReminderView.as_view()(request, **kwargs)
        assert response.status_code == 200

        request = rf.post("/")
        request.user = account_user
        response = OrgUserReminderView.as_view()(request, **kwargs)
        assert response.status_code == 302


class TestSignupView:
    def test_anon_user_can_access_signup_view(self, rf):
        """"""
        request = request_factory_login(rf)
        assert base.OrganizationSignup.as_view()(request).status_code == 200

    def test_authenticated_user_is_redirected_from_signup_view(self, rf, account_user):
        request = request_factory_login(rf, account_user)
        assert base.OrganizationSignup.as_view()(request).status_code == 302

    def test_anon_user_signup_base_class_has_no_success_url(self, rf):
        """"""
        request = request_factory_login(
            rf,
            method="post",
            path="/",
            data={
                "name": "An Association of Very Interesting People",
                "slug": "people",
                "email": "hey@people.org",
            },
        )
        with pytest.raises(ImproperlyConfigured):
            base.OrganizationSignup.as_view()(request)

    def test_anon_user_can_signup(self, rf):
        """"""

        class SignupView(base.OrganizationSignup):
            success_url = "/"

        request = request_factory_login(
            rf,
            method="post",
            path="/",
            data={
                "name": "An Association of Very Interesting People",
                "slug": "people",
                "email": "hey@people.org",
            },
        )
        response = SignupView.as_view()(request)
        assert response.status_code == 302

        # Verify its in the database
        Organization.objects.get(slug="an-association-of-very-interesting-people", is_active=False)


class TestBaseCreateOrganization:
    def test_get_org_create_view(self, rf):
        request = request_factory_login(rf)
        assert base.BaseOrganizationCreate.as_view()(request).status_code == 200

    def test_create_new_org(self, rf, account_user):
        request = request_factory_login(
            rf,
            user=account_user,
            method="post",
            path="/",
            data={"name": "Vizsla Club", "slug": "vizsla", "email": "hey@woof.org"},
        )
        response = base.BaseOrganizationCreate.as_view()(request)
        assert response.status_code == 302
        assert response["Location"] == reverse("organization_list")
        assert Organization.objects.get(slug="vizsla-club")


class TestBaseOrganizationDelete:
    def test_get_org_delete(self, rf, org_organization, account_user):
        request = request_factory_login(rf, user=account_user)
        kwargs = {"organization_pk": org_organization.pk}
        response = base.BaseOrganizationDelete.as_view()(request, **kwargs)
        assert response.status_code == 200

    def test_delete_organization_with_post(self, rf, org_organization, account_user):
        request = request_factory_login(rf, user=account_user, method="post", path="/")
        kwargs = {"organization_pk": org_organization.pk}
        response = base.BaseOrganizationDelete.as_view()(request, **kwargs)
        assert response.status_code == 302
        with pytest.raises(ObjectDoesNotExist):
            org_organization.refresh_from_db()


class TestBaseOrganizationUserDelete:
    def test_get_org_user_delete(
        self, rf, org_organization, account_user, extra_org_user
    ):
        request = request_factory_login(rf, user=account_user)
        kwargs = {"organization_pk": org_organization.pk, "user_pk": extra_org_user.pk}
        response = base.BaseOrganizationUserDelete.as_view()(request, **kwargs)
        assert response.status_code == 200

    def test_delete_organization_user_with_post(
        self, rf, org_organization, account_user, extra_org_user
    ):
        request = request_factory_login(rf, user=account_user, method="post", path="/")
        kwargs = {"organization_pk": org_organization.pk, "user_pk": extra_org_user.pk}
        response = base.BaseOrganizationUserDelete.as_view()(request, **kwargs)
        assert response.status_code == 302
        with pytest.raises(ObjectDoesNotExist):
            OrganizationUser.objects.get(user=extra_org_user)


@override_settings(USE_TZ=True)
class TestBasicOrgViews(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.kurt = User.objects.get(username="kurt")
        self.dave = User.objects.get(username="dave")
        self.dummy = User.objects.create_user(
            "dummy", email="dummy@example.com", password="test"
        )
        self.nirvana = Organization.objects.get(name="Nirvana")
        self.factory = RequestFactory()
        self.kurt_request = request_factory_login(self.factory, self.kurt)
        self.dave_request = request_factory_login(self.factory, self.dave)
        self.anon_request = request_factory_login(self.factory)

    def test_org_list(self):
        """Ensure that the status code 200 is returned"""
        self.assertEqual(
            200,
            base.BaseOrganizationList(request=self.kurt_request)
            .get(self.kurt_request)
            .status_code,
        )
        self.assertEqual(
            200,
            base.BaseOrganizationList(request=self.dave_request)
            .get(self.dave_request)
            .status_code,
        )

    def test_org_list_queryset(self):
        """Ensure only active organizations belonging to the user are listed"""
        self.assertEqual(
            1,
            base.BaseOrganizationList(request=self.kurt_request).get_queryset().count(),
        )
        self.assertEqual(
            2,
            base.BaseOrganizationList(request=self.dave_request).get_queryset().count(),
        )

    def test_org_detail(self):
        kwargs = {"organization_pk": self.nirvana.pk}
        self.assertEqual(
            200,
            base.BaseOrganizationDetail(request=self.kurt_request, kwargs=kwargs)
            .get(self.kurt_request, **kwargs)
            .status_code,
        )

    def test_org_update(self):
        kwargs = {"organization_pk": self.nirvana.pk}
        self.assertEqual(
            200,
            base.BaseOrganizationUpdate(request=self.kurt_request, kwargs=kwargs)
            .get(self.kurt_request, **kwargs)
            .status_code,
        )

    def test_user_list(self):
        kwargs = {"organization_pk": self.nirvana.pk}
        self.assertEqual(
            200,
            base.BaseOrganizationUserList(request=self.kurt_request, kwargs=kwargs)
            .get(self.kurt_request, **kwargs)
            .status_code,
        )

    def test_user_detail(self):
        kwargs = {"organization_pk": self.nirvana.pk, "user_pk": self.kurt.pk}
        self.assertEqual(
            200,
            base.BaseOrganizationUserDetail(request=self.kurt_request, kwargs=kwargs)
            .get(self.kurt_request, **kwargs)
            .status_code,
        )

    def test_bad_user_detail(self):
        kwargs = {"organization_pk": self.nirvana.pk, "user_pk": self.dummy.pk}
        self.assertRaises(
            Http404,
            base.BaseOrganizationUserDetail(
                request=self.kurt_request, kwargs=kwargs
            ).get,
            self.kurt_request,
            **kwargs
        )

    def test_user_create_get(self):
        kwargs = {"organization_pk": self.nirvana.pk}
        self.assertEqual(
            200,
            base.BaseOrganizationUserCreate(request=self.kurt_request, kwargs=kwargs)
            .get(self.kurt_request, **kwargs)
            .status_code,
        )

    def test_user_create_post(self):
        request = request_factory_login(
            self.factory,
            self.kurt,
            path="/",
            method="post",
            data={"email": "roadie@yahoo.com"},
        )
        kwargs = {"organization_pk": self.nirvana.pk}
        self.assertEqual(
            302,
            base.BaseOrganizationUserCreate.as_view()(request, **kwargs).status_code,
        )

    def test_user_update(self):
        kwargs = {"organization_pk": self.nirvana.pk, "user_pk": self.kurt.pk}
        self.assertEqual(
            200,
            base.BaseOrganizationUserUpdate(request=self.kurt_request, kwargs=kwargs)
            .get(self.kurt_request, **kwargs)
            .status_code,
        )



from django.contrib.auth.models import User
from django.test import TestCase
from django.test.utils import override_settings

from mock import call
from mock_django.signals import mock_signal_receiver

from organizations.models import Organization
from organizations.signals import owner_changed
from organizations.signals import user_added
from organizations.signals import user_removed


@override_settings(USE_TZ=True)
class SignalsTestCase(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.kurt = User.objects.get(username="kurt")
        self.dave = User.objects.get(username="dave")
        self.krist = User.objects.get(username="krist")
        self.duder = User.objects.get(username="duder")
        self.foo = Organization.objects.get(name="Foo Fighters")
        self.org = Organization.objects.get(name="Nirvana")
        self.admin = self.org.organization_users.get(user__username="krist")
        self.owner = self.org.organization_users.get(user__username="kurt")

    def test_user_added_called(self):

        with mock_signal_receiver(user_added) as add_receiver:
            self.foo.add_user(self.krist)

            self.assertEqual(
                add_receiver.call_args_list,
                [call(signal=user_added, sender=self.foo, user=self.krist)],
            )

        with mock_signal_receiver(user_added) as add_receiver:
            self.foo.get_or_add_user(self.duder)

            self.assertEqual(
                add_receiver.call_args_list,
                [call(signal=user_added, sender=self.foo, user=self.duder)],
            )

    def test_user_added_not_called(self):

        with mock_signal_receiver(user_added) as add_receiver:
            self.foo.get_or_add_user(self.dave)

            self.assertEqual(add_receiver.call_args_list, [])

    def test_user_removed_called(self):

        with mock_signal_receiver(user_removed) as remove_receiver:
            self.foo.add_user(self.krist)
            self.foo.remove_user(self.krist)

            self.assertEqual(
                remove_receiver.call_args_list,
                [call(signal=user_removed, sender=self.foo, user=self.krist)],
            )

    def test_owner_changed_called(self):

        with mock_signal_receiver(owner_changed) as changed_receiver:
            self.org.change_owner(self.admin)

            self.assertEqual(
                changed_receiver.call_args_list,
                [
                    call(
                        signal=owner_changed,
                        sender=self.org,
                        old=self.owner,
                        new=self.admin,
                    )
                ],
            )



from django.contrib.auth.models import User
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings

from organizations.models import Organization
from organizations.models import OrganizationUser
from organizations.views.mixins import AdminRequiredMixin
from organizations.views.mixins import MembershipRequiredMixin
from organizations.views.mixins import OrganizationMixin
from organizations.views.mixins import OrganizationUserMixin
from organizations.views.mixins import OwnerRequiredMixin
from tests.utils import request_factory_login


class ViewStub:
    def __init__(self, **kwargs):
        self.kwargs = kwargs

    def get_context_data(self, **kwargs):
        return kwargs

    def dispatch(self, request, *args, **kwargs):
        return HttpResponse("Success")


class OrgView(OrganizationMixin, ViewStub):
    """A testing view class"""

    pass


class UserView(OrganizationUserMixin, ViewStub):
    """A testing view class"""

    pass


@override_settings(USE_TZ=True)
class ObjectMixinTests(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.foo = Organization.objects.get(name="Foo Fighters")
        self.dave = OrganizationUser.objects.get(
            user__username="dave", organization=self.foo
        )

    def test_get_org_object(self):
        view = OrgView(organization_pk=self.foo.pk)
        self.assertEqual(view.get_object(), self.foo)

    def test_get_user_object(self):
        view = UserView(organization_pk=self.foo.pk, user_pk=self.dave.pk)
        self.assertEqual(view.get_object(), self.dave)
        self.assertEqual(view.get_organization(), self.foo)

    def test_get_model(self):
        """Ensure that the method returns the class object"""
        self.assertEqual(Organization, OrganizationMixin().get_org_model())
        self.assertEqual(Organization, OrganizationUserMixin().get_org_model())
        self.assertEqual(OrganizationUser, OrganizationUserMixin().get_user_model())


@override_settings(USE_TZ=True)
class AccessMixinTests(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.nirvana = Organization.objects.get(name="Nirvana")
        self.kurt = User.objects.get(username="kurt")
        self.krist = User.objects.get(username="krist")
        self.dave = User.objects.get(username="dave")
        self.dummy = User.objects.create_user(
            "dummy", email="dummy@example.com", password="test"
        )
        self.factory = RequestFactory()
        self.kurt_request = request_factory_login(self.factory, self.kurt)
        self.krist_request = request_factory_login(self.factory, self.krist)
        self.dave_request = request_factory_login(self.factory, self.dave)
        self.dummy_request = request_factory_login(self.factory, self.dummy)

    def test_member_access(self):
        class MemberView(MembershipRequiredMixin, OrgView):
            pass

        self.assertEqual(
            200,
            MemberView()
            .dispatch(self.kurt_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        self.assertEqual(
            200,
            MemberView()
            .dispatch(self.krist_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        self.assertEqual(
            200,
            MemberView()
            .dispatch(self.dave_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        with self.assertRaises(PermissionDenied):
            MemberView().dispatch(self.dummy_request, organization_pk=self.nirvana.pk)

    def test_admin_access(self):
        class AdminView(AdminRequiredMixin, OrgView):
            pass

        self.assertEqual(
            200,
            AdminView()
            .dispatch(self.kurt_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        self.assertEqual(
            200,
            AdminView()
            .dispatch(self.krist_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        # Superuser
        self.assertEqual(
            200,
            AdminView()
            .dispatch(self.dave_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        with self.assertRaises(PermissionDenied):
            AdminView().dispatch(self.dummy_request, organization_pk=self.nirvana.pk)

    def test_owner_access(self):
        class OwnerView(OwnerRequiredMixin, OrgView):
            pass

        self.assertEqual(
            200,
            OwnerView()
            .dispatch(self.kurt_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        with self.assertRaises(PermissionDenied):
            OwnerView().dispatch(self.krist_request, organization_pk=self.nirvana.pk)
        # Superuser
        self.assertEqual(
            200,
            OwnerView()
            .dispatch(self.dave_request, organization_pk=self.nirvana.pk)
            .status_code,
        )
        with self.assertRaises(PermissionDenied):
            OwnerView().dispatch(self.dummy_request, organization_pk=self.nirvana.pk)



"""
Tests for configurable fields
"""
import importlib

from django.core.exceptions import ImproperlyConfigured

import pytest

from organizations import fields


def test_misconfigured_autoslug_cannot_import(settings):
    settings.ORGS_SLUGFIELD = "not.AModel"
    with pytest.raises(ImproperlyConfigured):
        importlib.reload(fields)


def test_misconfigured_autoslug_incorrect_class(settings):
    settings.ORGS_SLUGFIELD = "autoslug.AutoSlug"
    with pytest.raises(ImproperlyConfigured):
        importlib.reload(fields)


def test_misconfigured_autoslug_bad_notation(settings):
    settings.ORGS_SLUGFIELD = "autoslug.AutoSlugField"
    importlib.reload(fields)



from functools import partial

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.utils import override_settings

from organizations.models import Organization
from organizations.utils import create_organization
from organizations.utils import model_field_attr
from test_abstract.models import CustomOrganization
from test_accounts.models import Account


@override_settings(USE_TZ=True)
class CreateOrgTests(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.user = User.objects.get(username="dave")

    def test_create_organization(self):
        acme = create_organization(
            self.user, "Acme", org_defaults={"slug": "acme-slug"}
        )
        self.assertTrue(isinstance(acme, Organization))
        self.assertEqual(self.user, acme.owner.organization_user.user)
        self.assertTrue(acme.owner.organization_user.is_admin)

    def test_create_custom_org(self):
        custom = create_organization(self.user, "Custom", model=Account)
        self.assertTrue(isinstance(custom, Account))
        self.assertEqual(self.user, custom.owner.organization_user.user)

    def test_create_custom_org_from_abstract(self):
        custom = create_organization(self.user, "Custom", model=CustomOrganization)
        self.assertTrue(isinstance(custom, CustomOrganization))
        self.assertEqual(self.user, custom.owner.organization_user.user)

    def test_defaults(self):
        """Ensure models are created with defaults as specified"""
        # Default models
        org = create_organization(
            self.user,
            "Is Admin",
            org_defaults={"slug": "is-admin-212", "is_active": False},
            org_user_defaults={"is_admin": False},
        )
        self.assertFalse(org.is_active)
        self.assertFalse(org.owner.organization_user.is_admin)

        # Custom models
        create_account = partial(
            create_organization,
            model=Account,
            org_defaults={"monthly_subscription": 99},
            org_user_defaults={"user_type": "B"},
        )
        myaccount = create_account(self.user, name="My New Account")
        self.assertEqual(myaccount.monthly_subscription, 99)

    def test_backwards_compat(self):
        """Ensure old optional arguments still work"""
        org = create_organization(self.user, "Is Admin", "my-slug", is_active=False)
        self.assertFalse(org.is_active)

        custom = create_organization(self.user, "Custom org", org_model=Account)
        self.assertTrue(isinstance(custom, Account))


class AttributeUtilTests(TestCase):
    def test_present_field(self):
        self.assertTrue(model_field_attr(User, "username", "max_length"))

    def test_absent_field(self):
        self.assertRaises(KeyError, model_field_attr, User, "blahblah", "max_length")

    def test_absent_attr(self):
        self.assertRaises(
            AttributeError, model_field_attr, User, "username", "mariopoints"
        )



# Generated by Django 4.0.7 on 2022-09-30 17:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("organizations", "0004_organizationinvitation"),
    ]

    operations = [
        migrations.AlterField(
            model_name="organization",
            name="users",
            field=models.ManyToManyField(
                related_name="%(app_label)s_%(class)s",
                through="organizations.OrganizationUser",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="organizationinvitation",
            name="invited_by",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_sent_invitations",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="organizationinvitation",
            name="invitee",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_invitations",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="organizationuser",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



"""
Tests that migrations are not missing
"""

from django.core.management import call_command

import pytest


@pytest.mark.django_db
def test_no_missing_migrations():
    """Check no model changes have been made since the last `./manage.py makemigrations`.
    """
    call_command("makemigrations", check=True, dry_run=True)



from django.contrib.auth.models import User
from django.template import Context
from django.template import Template
from django.test import TestCase
from django.test.utils import override_settings

from organizations.models import Organization


@override_settings(USE_TZ=True)
class TestTagsAndFilters(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.kurt = User.objects.get(username="kurt")
        self.dave = User.objects.get(username="dave")
        self.nirvana = Organization.objects.get(name="Nirvana")
        self.foo = Organization.objects.get(name="Foo Fighters")
        self.context = {}

    def test_organization_users_tag(self):
        self.context = {"organization": self.nirvana}
        out = Template(
            "{% load org_tags %}" "{% organization_users organization %}"
        ).render(Context(self.context))
        self.assertIn("Kurt", out)
        self.assertIn("Dave", out)

    def test_is_owner_org_filter(self):
        self.context = {"organization": self.nirvana, "user": self.kurt}
        out = Template(
            "{% load org_tags %}"
            "{% if organization|is_owner:user %}"
            "Is Owner"
            "{% endif %}"
        ).render(Context(self.context))
        self.assertEqual(out, "Is Owner")

    def test_is_admin_org_filter(self):
        self.context = {"organization": self.foo, "user": self.dave}
        out = Template(
            "{% load org_tags %}"
            "{% if organization|is_admin:user %}"
            "Is Admin"
            "{% endif %}"
        ).render(Context(self.context))
        self.assertEqual(out, "Is Admin")

    def test_is_not_admin_org_filter(self):
        self.context = {"organization": self.nirvana, "user": self.dave}
        out = Template(
            "{% load org_tags %}"
            "{% if not organization|is_admin:user %}"
            "Is Not Admin"
            "{% endif %}"
        ).render(Context(self.context))
        self.assertEqual(out, "Is Not Admin")



# Generated by Django 4.0.7 on 2022-10-13 02:45

from django.db import migrations
import organizations.fields


class Migration(migrations.Migration):
    dependencies = [
        ("organizations", "0005_alter_organization_users_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="organization",
            name="slug",
            field=organizations.fields.SlugField(
                blank=True,
                editable=False,
                help_text="The name in all lowercase, suitable for URL identification",
                max_length=200,
                populate_from="name",
                unique=True,
            ),
        ),
    ]



from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings

from organizations.forms import OrganizationAddForm
from organizations.forms import OrganizationForm
from organizations.forms import OrganizationUserAddForm
from organizations.forms import OrganizationUserForm
from organizations.models import Organization
from tests.utils import request_factory_login

User = get_user_model()


class TestOrganizationAddForm(TestCase):
    """
    Tests for adding new organizations
    """

    def setUp(self):
        self.factory = RequestFactory()

    def test_expected_valid_data_validates(self):
        """Test our happy path"""
        request = self.factory.request()
        form = OrganizationAddForm(
            request,
            data={"slug": "new_org", "name": "New Org", "email": "cthulu@oldgods.org"},
        )
        self.assertTrue(form.is_valid())

    def test_add_organization_for_existing_user(self):
        user = User.objects.create_user(
            "timmy", password="ajsdkfa3", email="timmy@whoa.com"
        )
        request = self.factory.request()
        form = OrganizationAddForm(
            request, data={"slug": "new_org", "name": "New Org", "email": user.email}
        )
        self.assertTrue(form.is_valid())
        new_org = form.save()
        self.assertTrue(new_org.is_active)
        self.assertEqual(new_org.name, "New Org")

    def test_add_organization_for_new_user(self):
        user = User.objects.create_user(
            "timmy", password="ajsdkfa3", email="timmy@whoa.com"
        )
        request = request_factory_login(self.factory, user)
        form = OrganizationAddForm(
            request,
            data={
                "slug": "new_org",
                "name": "New Org",
                "email": "i.am.new.here@geemail.com",
            },
        )
        self.assertTrue(form.is_valid())
        new_org = form.save()
        self.assertFalse(new_org.is_active)  # Inactive until confirmation


class TestOrganizationUserAddForm(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.factory = RequestFactory()
        self.org = Organization.objects.get(name="Nirvana")
        self.owner = self.org.organization_users.get(user__username="kurt")

    def test_multiple_users_exist(self):
        User.objects.create_user("asdkjf", password="ajsdkfa", email="bob@bob.com")
        User.objects.create_user("asdkjf1", password="ajsdkfa3", email="bob@bob.com")
        request = request_factory_login(self.factory, self.owner.user)
        form = OrganizationUserAddForm(
            request=request, organization=self.org, data={"email": "bob@bob.com"},
        )
        self.assertFalse(form.is_valid())

    def test_add_user_already_in_organization(self):
        admin = self.org.organization_users.get(user__username="krist")
        request = request_factory_login(self.factory, self.owner.user)
        form = OrganizationUserAddForm(
            request=request, organization=self.org, data={"email": admin.user.email},
        )
        self.assertFalse(form.is_valid())

    def test_save_org_user_add_form(self):
        request = request_factory_login(self.factory, self.owner.user)
        form = OrganizationUserAddForm(
            request=request,
            organization=self.org,
            data={"email": "test_email@example.com", "is_admin": False},
        )
        self.assertTrue(form.is_valid())
        form.save()


@override_settings(USE_TZ=True)
class TestOrganizationForm(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.factory = RequestFactory()
        self.org = Organization.objects.get(name="Nirvana")
        self.admin = self.org.organization_users.get(user__username="krist")
        self.owner = self.org.organization_users.get(user__username="kurt")

    def test_admin_edits_org(self):
        user = self.admin.user
        request = request_factory_login(self.factory, user)
        form = OrganizationForm(
            request,
            instance=self.org,
            data={"name": self.org.name, "slug": self.org.slug, "owner": self.owner.pk},
        )
        self.assertTrue(form.is_valid())
        form = OrganizationForm(
            request,
            instance=self.org,
            data={"name": self.org.name, "slug": self.org.slug, "owner": self.admin.pk},
        )
        self.assertFalse(form.is_valid())

    def test_owner_edits_org(self):
        user = self.owner.user
        request = request_factory_login(self.factory, user)
        form = OrganizationForm(
            request,
            instance=self.org,
            data={"name": self.org.name, "slug": self.org.slug, "owner": self.owner.pk},
        )
        self.assertTrue(form.is_valid())
        form = OrganizationForm(
            request,
            instance=self.org,
            data={"name": self.org.name, "slug": self.org.slug, "owner": self.admin.pk},
        )
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEqual(self.org.owner.organization_user, self.admin)


class TestOrganizationUserForm(TestCase):
    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.factory = RequestFactory()
        self.org = Organization.objects.get(name="Nirvana")
        self.admin = self.org.organization_users.get(user__username="krist")
        self.owner = self.org.organization_users.get(user__username="kurt")

    def test_edit_owner_user(self):
        form = OrganizationUserForm(instance=self.owner, data={"is_admin": True})
        self.assertTrue(form.is_valid())
        form = OrganizationUserForm(instance=self.owner, data={"is_admin": False})
        self.assertFalse(form.is_valid())

    def test_save_org_form(self):
        request = request_factory_login(self.factory, self.owner.user)
        form = OrganizationForm(
            request,
            instance=self.org,
            data={"name": self.org.name, "slug": self.org.slug, "owner": self.owner.pk},
        )
        self.assertTrue(form.is_valid())
        form.save()

    def test_save_user_form(self):
        form = OrganizationUserForm(instance=self.owner, data={"is_admin": True})
        self.assertTrue(form.is_valid())
        form.save()



from functools import partial

from django.contrib.auth.models import User
from django.db import IntegrityError
from django.test import TestCase
from django.test.utils import override_settings

from organizations.models import Organization
from organizations.models import OrganizationInvitation
from organizations.models import OrganizationOwner
from organizations.models import OrganizationUser
from organizations.utils import create_organization
from test_abstract.models import CustomOrganization
from test_accounts.models import Account
from test_accounts.models import AccountInvitation
from test_custom.models import Team


@override_settings(USE_TZ=True)
class ActiveManagerTests(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def test_active(self):
        self.assertEqual(3, Organization.objects.all().count())
        self.assertEqual(2, Organization.active.all().count())

    def test_by_user(self):
        user = User.objects.get(username="dave")
        self.assertEqual(3, Organization.objects.get_for_user(user).count())
        self.assertEqual(2, Organization.active.get_for_user(user).count())


@override_settings(USE_TZ=True)
class OrgModelTests(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.kurt = User.objects.get(username="kurt")
        self.dave = User.objects.get(username="dave")
        self.krist = User.objects.get(username="krist")
        self.duder = User.objects.get(username="duder")
        self.nirvana = Organization.objects.get(name="Nirvana")
        self.foo = Organization.objects.get(name="Foo Fighters")

    def test_invitation_model(self):
        assert Organization.invitation_model == OrganizationInvitation

    def test_org_string_representation(self):
        """Ensure that models' string representation are error free"""
        self.foo.name = "Föö Fíghterß"
        self.assertTrue("{0}".format(self.foo))
        self.assertTrue("{0}".format(self.foo.owner))
        self.assertTrue("{0}".format(self.foo.owner.organization_user))

    def test_relation_name(self):
        """Ensure user-related name is accessible from common attribute"""
        self.assertEqual(self.foo.user_relation_name, "organizations_organization")

    def test_duplicate_members(self):
        """Ensure that a User can only have one OrganizationUser object"""
        self.assertRaises(IntegrityError, self.nirvana.add_user, self.dave)

    def test_is_member(self):
        self.assertTrue(self.nirvana.is_member(self.kurt))
        self.assertTrue(self.nirvana.is_member(self.dave))
        self.assertTrue(self.foo.is_member(self.dave))
        self.assertFalse(self.foo.is_member(self.kurt))

    def test_is_admin(self):
        self.assertTrue(self.nirvana.is_admin(self.kurt))
        self.assertTrue(self.nirvana.is_admin(self.krist))
        self.assertFalse(self.nirvana.is_admin(self.dave))
        self.assertTrue(self.foo.is_admin(self.dave))

    def test_is_owner(self):
        self.assertTrue(self.nirvana.is_owner(self.kurt))
        self.assertTrue(self.foo.is_owner(self.dave))
        self.assertFalse(self.nirvana.is_owner(self.dave))
        self.assertFalse(self.nirvana.is_owner(self.krist))

    def test_add_user(self):
        new_guy = self.foo.add_user(self.krist)
        self.assertTrue(isinstance(new_guy, OrganizationUser))
        self.assertEqual(new_guy.organization, self.foo)

    def test_remove_user(self):
        self.foo.add_user(self.krist)
        self.foo.remove_user(self.krist)
        self.assertFalse(self.foo.users.filter(pk=self.krist.pk).exists())

    def test_get_or_add_user(self):
        """Ensure `get_or_add_user` adds a user IFF it exists"""
        new_guy, created = self.foo.get_or_add_user(self.duder)
        self.assertTrue(isinstance(new_guy, OrganizationUser))
        self.assertEqual(new_guy.organization, self.foo)
        self.assertTrue(created)

        new_guy, created = self.foo.get_or_add_user(self.dave)
        self.assertTrue(isinstance(new_guy, OrganizationUser))
        self.assertFalse(created)

    def test_delete_owner(self):
        from organizations.exceptions import OwnershipRequired

        owner = self.nirvana.owner.organization_user
        self.assertRaises(OwnershipRequired, owner.delete)

    def test_change_owner(self):
        admin = self.nirvana.organization_users.get(user__username="krist")
        self.nirvana.change_owner(admin)
        owner = self.nirvana.owner.organization_user
        self.assertEqual(owner, admin)

    def test_delete_missing_owner(self):
        """Ensure an org user can be deleted when there is no owner"""
        org = Organization.objects.create(name="Some test", slug="some-test")
        # Avoid the Organization.add_user method which would make an owner
        org_user = OrganizationUser.objects.create(user=self.kurt, organization=org)
        # Just make sure it doesn't raise an error
        org_user.delete()

    def test_nonmember_owner(self):
        from organizations.exceptions import OrganizationMismatch

        foo_user = self.foo.owner
        self.nirvana.owner = foo_user
        self.assertRaises(OrganizationMismatch, self.nirvana.owner.save)


@override_settings(USE_TZ=True)
class OrgDeleteTests(TestCase):

    fixtures = ["users.json", "orgs.json"]

    def test_delete_account(self):
        """Ensure Users are not deleted on the cascade"""
        self.assertEqual(3, OrganizationOwner.objects.all().count())
        self.assertEqual(4, User.objects.all().count())
        scream = Organization.objects.get(name="Scream")
        scream.delete()
        self.assertEqual(2, OrganizationOwner.objects.all().count())
        self.assertEqual(4, User.objects.all().count())

    def test_delete_orguser(self):
        """Ensure the user is not deleted on the cascade"""
        krist = User.objects.get(username="krist")
        org_user = OrganizationUser.objects.filter(
            organization__name="Nirvana", user=krist
        )
        org_user.delete()
        self.assertTrue(krist.pk)


class CustomModelTests(TestCase):

    # Load the world as we know it.
    fixtures = ["users.json", "orgs.json"]

    def setUp(self):
        self.kurt = User.objects.get(username="kurt")
        self.dave = User.objects.get(username="dave")
        self.krist = User.objects.get(username="krist")
        self.duder = User.objects.get(username="duder")
        self.red_account = Account.objects.create(
            name="Red Account", monthly_subscription=1200
        )

    def test_invitation_model(self):
        assert Account.invitation_model == AccountInvitation

    def test_org_string(self):
        self.assertEqual(self.red_account.__str__(), "Red Account")

    def test_relation_name(self):
        """Ensure user-related name is accessible from common attribute"""
        self.assertEqual(self.red_account.user_relation_name, "test_accounts_account")

    def test_change_user(self):
        """Ensure custom organizations validate in owner change"""
        create_team = partial(create_organization, model=Team)
        hometeam = create_team(self.dave, "Hometeam")
        duder_org_user = hometeam.add_user(self.duder)
        hometeam.owner.organization_user = duder_org_user
        hometeam.owner.save()

    def test_abstract_change_user(self):
        """
        Ensure custom organizations inheriting abstract model
        validate in owner change
        """
        create_org = partial(create_organization, model=CustomOrganization)
        org1 = create_org(self.dave, "Org1")
        duder_org_user = org1.add_user(self.duder)
        org1.owner.organization_user = duder_org_user
        org1.owner.save()
